/* $Id: ast_andthen.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_ANDTHEN_H
#define WTPARSER_AST_ANDTHEN_H

#include "../astnode.h"
#include "ast_expression.h"

namespace wtast
{

class Type;

class AndThen : public Expression
{
	Expression* mLHS;
	Expression* mRHS;
	Type*       mType;

public:
	AndThen(Expression* lhs, Expression* rhs, Type* type)
		: Expression()
		, mLHS(lhs)
		, mRHS(rhs)
		, mType(type)
	{
		// ok
	}

	~AndThen();

	void Accept(AstVisitor* vis);

	AndThen* Copy();

	Kind GetKind() const
	{
		return ANDTHEN;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return Variable::READONLY==varclass || Variable::LOCAL==varclass;
	}

	Expression* GetLeftHandSide()
	{
		return mLHS;
	}

	Expression* GetRightHandSide()
	{
		return mRHS;
	}

	TypeInfo* GetType();
};

} // namespace wtast

#endif // WTPARSER_AST_ANDTHEN_H
