/* $Id: ast_functiondefinition.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_functiondefinition.h"
#include "ast_functionsignature.h"
#include "ast_precondition.h"
#include "ast_postcondition.h"
#include "ast_exceptioncompound.h"
#include "../astvisitor.h"

wtast::FunctionDefinition::~FunctionDefinition()
{
	delete mFuncSig;
	delete mPreCond;
	delete mPostCond;
	delete mBody;
}

void wtast::FunctionDefinition::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		mFuncSig->Accept(vis);
		if(mPreCond) { mPreCond->Accept(vis); }
		if(mPostCond) { mPostCond->Accept(vis); }
		mBody->Accept(vis);
		vis->EndVisit(this);
	}
}

wtast::FunctionDefinition* wtast::FunctionDefinition::Copy()
{
	PreCondition* precond  = 0;
	PostCondition* postcond = 0;

	if(mPreCond) { precond = mPreCond->Copy(); }
	if(mPostCond) { postcond = mPostCond->Copy(); }

	return new FunctionDefinition(mFuncSig->Copy(), precond, postcond, mBody->Copy());
}
