/* $Id: ast_memberaccess.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_MEMBERACCESS_H
#define WTPARSER_AST_MEMBERACCESS_H

#include "../astnode.h"
#include "ast_lvalue.h"
#include "ast_type.h"
#include "welltype/wtruntime.h"
#include <string>

namespace wtast
{

class Expression;

class MemberAccess : public LValue
{
	LValue*     mLValue;
	std::string mFieldName;
	TypeInfo*   mResultType;
	wt_u32      mOffset;

public:
	MemberAccess(LValue* lvalue, const std::string & fieldname, TypeInfo* result_type, wt_u32 offset)
		: LValue()
		, mLValue(lvalue)
		, mFieldName(fieldname)
		, mResultType(result_type)
		, mOffset(offset)
	{
		// ok
	}

	~MemberAccess();

	void Accept(AstVisitor* vis);

	MemberAccess* Copy();

	Kind GetKind() const
	{
		return MEMBERACCESS;
	}

	LValue* GetLValue()
	{
		return mLValue;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return mLValue->OriginateFrom(varclass);
	}

	const std::string & GetFieldName() const
	{
		return mFieldName;
	}

	TypeInfo* GetType()
	{
		return mResultType;
	}

	wt_u32 GetOffset()
	{
		return mOffset;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_MEMBERACCESS_H
