//================================================================================================
// $Id: lex.yy.cc.h 4611 2015-11-19 16:42:49Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		lex.yy.cc extension
// Authors:
//		Aron Barath, 2014-2015
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_LEX_YY_CC_H
#define WTPARSER_LEX_YY_CC_H

#include "parser_pre.h"
#include "frontend.h"
#undef yyFlexLexer
#define yyFlexLexer yyWTFlexLexer
#include <FlexLexer.h>
#include <string>

namespace wtparser
{
class ParserHelper;
class ProblemHandler;
} // namespace wtparser

class WTFlexLexer : public yyWTFlexLexer
{
public:
	WTFlexLexer(wtparser::ParserHelper* h, wtparser::IFile* file, WTFlexLexer* prev);
	virtual ~WTFlexLexer() { }
	WTFlexLexer* GetPrevious() { return mPrevious; }
	unsigned int GetStartLine() const { return mStartPosInfo.line; }
	unsigned int GetStartColumn() const { return mStartPosInfo.col; }
	unsigned int GetEndLine() const { return mEndPosInfo.line; }
	unsigned int GetEndColumn() const { return mEndPosInfo.col; }
	wtparser::ParserHelper* GetHelper() { return mHelper; }

	friend void WTFlexLexer_UpdatePositionInfo(WTFlexLexer* lexer, unsigned int len);
	friend void WTFlexLexer_NewLine(WTFlexLexer* lexer);
	friend void WTFlexLexer_WhiteSpaces(WTFlexLexer* lexer, const char* str, unsigned int len);

protected:
	virtual int LexerInput(char* buf, int max_size)
	{
		return mFile->Read(buf, max_size);
	}

	virtual void LexerOutput(const char* buf, int size);
	virtual void LexerError(const char* msg);

private:
	struct PositionInfo
	{
		PositionInfo() : line(1), col(1) { }

		unsigned int line;
		unsigned int col;
	};

private:
	wtparser::IFile*           mFile;
	wtparser::ProblemHandler & mProbHan;
	WTFlexLexer*               mPrevious;
	wtparser::ParserHelper*    mHelper;
	unsigned int               mColumn;
	PositionInfo               mStartPosInfo;
	PositionInfo               mEndPosInfo;
};

void WTFlexLexer_UpdatePositionInfo(WTFlexLexer* lexer, unsigned int len);
void WTFlexLexer_NewLine(WTFlexLexer* lexer);
void WTFlexLexer_WhiteSpaces(WTFlexLexer* lexer, const char* str, unsigned int len);

#endif // WTPARSER_LEX_YY_CC_H
