#ifndef WTRUNTIME_OBJECTS_H
#define WTRUNTIME_OBJECTS_H

#include "config.h"
#include "types.h"
#include "debug.h"
#include "program.h"

#ifdef ADD_INLINE_OBJECT_MAGIC
#  define WTR_MAGIC_STR1        0x57571010
#  define WTR_MAGIC_STR2        0xE5658356
#  define WTR_MAGIC_SEQ1        0x5E5E2020
#  define WTR_MAGIC_SEQ2        0xE5E56717
#  define WTR_MAGIC_REC1        0xECEC3030
#  define WTR_MAGIC_REC2        0xEEC61587
#endif /* ADD_INLINE_OBJECT_MAGIC */

struct wtr_vm;

struct wtr_str
{
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic1;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 length;  /* length of the string */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic2;
	wt_u32 length_save;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	char   data[1]; /* content of the string (including the terminating zero) */
};

struct wtr_seq
{
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic1;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 length;   /* number of elements in the sequence */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic2;
	wt_u32 length_save;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 elemsize; /* size of an element in octets */
	wt_u8  data[1];  /* data area */
};

struct wtr_rec
{
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic1;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u32 size;   /* size of the record in octets */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wt_u32 magic2;
	wt_u32 size_save;
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wt_u8  data[1]; /* data area */
};


/*----------------------------------------------------------------*/


/* String (or character sequence). Always contains an additional */
/* terminating zero. The 'data' parameter is optional, it can be */
/* NULL, but in this case  The string data will be uninitialized. */
wt_vmptr wtr_str_create_z(struct wtr_vm* const vm, const char* data);
wt_vmptr wtr_str_create_n(struct wtr_vm* const vm, const char* data,
	wt_u32 len);
void wtr_str_delete(struct wtr_vm* const vm, wt_vmptr str);

/* Sequence. An indexable sequence which contains elements. */
/* The element size cannot be zero. */
/* The virtual machine does not interpret the structures, this is only a shell. */
wt_vmptr wtr_seq_create(struct wtr_vm* const vm, wt_u32 elemsize,
	wt_u32 count);
void wtr_seq_delete(struct wtr_vm* const vm, wt_vmptr seq);

/* Record. A plain dynamically allocated memory area. */
/* The virtual machine does not interpret the structures, this is only a shell. */
wt_vmptr wtr_rec_create(struct wtr_vm* const vm, wt_u32 size);
void wtr_rec_delete(struct wtr_vm* const vm, wt_vmptr rec);


#ifdef ADD_INLINE_OBJECT_MAGIC

/* Determines whether the string object is valid or not. */
/* Returns: true, if magic1 and magic2 is valid */
/*   str - string object */
wt_bool wtr_str_is_valid(struct wtr_vm* const vm, wt_vmptr p);


/* Determines whether the sequence object is valid or not. */
/* Returns: true, if magic1 and magic2 is valid */
/*   seq - sequence object */
wt_bool wtr_seq_is_valid(struct wtr_vm* const vm, wt_vmptr p);


/* Determines whether the record object is valid or not. */
/* Returns: true, if magic1 and magic2 is valid */
/*   rec - record object */
wt_bool wtr_rec_is_valid(struct wtr_vm* const vm, wt_vmptr p);

#endif /* ADD_INLINE_OBJECT_MAGIC */

/*----------------------------------------------------------------*/

#endif /* WTRUNTIME_OBJECTS_H */
