#ifndef WTRUNTIME_PROGRAM_H
#define WTRUNTIME_PROGRAM_H

#include "config.h"
#include "types.h"
#include "vector.h"
#include <stdlib.h>

struct wtr_vm;
struct wtr_program;
struct wtr_exception;
struct wtr_function;
struct wtr_type;
struct wtr_typeslot;
struct wtr_operator;

struct wtr_program
{
	struct wtr_node        link_programs;         /* link in the program list of the VM */
	struct wtr_vm*         vm;                    /* the program belongs to this virtual machine */
	wt_uintptr             refcount;              /* number of references to this program (due to other programs, or function handles) -- starts at 1 (self-reference) */
	wt_vmptr               program_data;          /* data allocated for the program (that the binary specified to the loader) */
	wt_uintptr             program_size;          /* size of 'program_data' in octets */
	void*                  user_ptr;              /* an additional user pointer, this is opaque to the VM */
	wt_u32                 pc_base;               /* program-local PC base address relative to program.code */
	wt_u32                 pc_ctor;               /* PC of the ctor{} (program-relative address; WTR_NULLPC if not exists */
	wt_u32                 pc_dtor;               /* PC of the dtor{} (program-relative address; WTR_NULLPC if not exists */
	wt_u32                 pc_main;               /* entry point of the program (in practice PC of main{}), program-relative address */
	wt_vmptr               base_address;          /* VM address of 'program_data' */
	wt_vmptr               typevec_offs;          /* offset of the program's type vector based to 'base_address' */
	size_t                 num_ref_types;         /* number of referenced types */
	size_t                 max_ref_types;         /* number of referenced types */
	struct wtr_typeslot*   referenced_types;      /* array of referenced types, allocated with system allocator */
	struct wtr_vector      referenced_functions;
	struct wtr_vector      referenced_exceptions;
	struct wtr_vector      own_exceptions;
	struct wtr_vector      referenced_operators;
	struct wtr_vector      own_operators;
	struct wtr_vector      own_types;
	struct wtr_vector      metadata_vec;
};

/* metadata structure, key string and value string are allocated in a */
/* single memory block, thus freeing a metadata is achive by calling */
/* wtr_vm_sys_free(vm, metadata_ptr) */
struct wtr_metadata
{
	const char* key;
	const char* val;
};


/* resolve a type in the context of the given program */
wt_bool wtr_program_resolvetype(struct wtr_program* prog, wt_u32* p_type);


void wtr_program_destroy(struct wtr_vm* const vm,
	struct wtr_program* const program);


/* free up referenced and owned objects and their containers -- can be used */
/* from the loader if 'ownfunc' is not NULL */
void wtr_program_free_refd_owned(struct wtr_vm* const vm,
	struct wtr_program* const program, struct wtr_vector* const ownfunc);


#endif /* WTRUNTIME_PROGRAM_H */
